#!/bin/bash
# 
# This is a tool which fixes a common database incompatibility 
# when upgrading JVerein from a version below 2.8.1 to any higher 
# version.
#
# It is related to a version upgrade of the H2 data base driver
# from 1.4.196 to higher versions.
#
# The strategy of this script is: fix the H2 database directly
# by exporting it using the old H2 driver and re-importing it
# using the new H2 driver.
#
# Algorithm is as follows:
# 
# 1. Make a copy of the Jameica installation (program, not data)
# 2. Replace the H2 driver with the old version
# 3. Load a working backup into JVerein
# 4. Export the contents of the H2 database to a SQL file
# 6. Import the contents from the SQL file into a new H2 database
# 7. Test with the actual Jameica installation
#
# The solution is taken from
# https://jverein-forum.de/viewtopic.php?f=5&t=4525&start=10#p17616
#

TARGET=~/.jameica
TARGET_INSTALL=/opt/jameica/
WORKDIR=$(mktemp -d ~/fix-jverein.XXXXXXXX)
function cleanup() {
  rm -rf ${WORKDIR}
}
trap "cleanup" EXIT # make sure that temp files are deleted


echo 
echo '######################################################################'
echo '### This script tries to fix your JVerein installation                '
echo '### after an upgrade from a version below 2.8.2                       '
echo '### when using the default H2 database                                '
echo '###                                                                   '
echo "### Jameica installation taken from: ${TARGET_INSTALL}"
echo "### JVerein data directory: ${TARGET}"
echo "### Working directory: ${WORKDIR}"
echo '######################################################################'
echo 
echo '######################################################################'
echo '### '
echo '###      WARNING: This script will modify your JVerein database'
echo '### '
echo '######################################################################'
echo 
read -p 'Hit <RETURN> to continue or <CTRL-C> to quit ...' dummy


LIBDIR=${WORKDIR}/libs   # This is where the H2 libraries go
BACKUP=${WORKDIR}/backup # This is where the database export goes
RUNDIR=${WORKDIR}/run    # A copy of the Jameica installation will be placed here
RUNH2DIR=${RUNDIR}/jameica/lib/h2

CURRENT_DATE=$(date +%Y%m%d-%H%M%S)

TARGET_DATA=${TARGET}/jverein/h2db
TARGET_DATA_BACKUP=${TARGET}/jverein/h2db.backup-${CURRENT_DATE}

H2_ORIG=`cd ${TARGET_INSTALL}/lib/h2; ls h2*.jar`
H2_LEGACY=h2-1.4.196.jar
H2_LEGACY_URL=http://insecure.repo1.maven.org/maven2/com/h2database/h2/1.4.196/${H2_LEGACY}
echo "Original H2 library: ${H2_ORIG}"
echo "Legacy H2 library: ${H2_LEGACY}"

# Create directories
mkdir -p ${WORKDIR}
mkdir -p ${RUNDIR}
mkdir -p ${LIBDIR}
mkdir -p ${BACKUP}


cd ${WORKDIR}

# 0. Fix JVerein 2.8.18 -> 2.8.17
# -------------------------------
wget http://www.jverein.de/updates/2.8/jverein.2.8.17.zip
zip -r jverein-plugin-backup.zip ${TARGET}/plugins/jverein
rm -r ${TARGET}/plugins/jverein
unzip jverein.2.8.17.zip -d ${TARGET}/plugins/


# Fix PDF generation (backport, if Jameica program is too new)
# https://jverein-forum.de/viewtopic.php?f=5&t=4585&p=15656&hilit=itext#p15656
wget http://insecure.repo1.maven.org/maven2/com/itextpdf/itextpdf/5.3.4/itextpdf-5.3.4.jar
mv itextpdf-5.3.4.jar ${TARGET}/plugins/hibiscus/lib

# 1. Make a copy of the Jameica installation (program, not data)
# --------------------------------------------------------------

cp -a ${TARGET_INSTALL} ${RUNDIR}


# 2. Replace the H2 driver with the old version
# --------------------------------------------

# Put legacy and original H2 library to LIBDIR
wget ${H2_LEGACY_URL}
mv ${H2_LEGACY} ${LIBDIR}
cp ${RUNH2DIR}/${H2_ORIG} ${LIBDIR}

# Replace H2 library 
cp ${LIBDIR}/${H2_LEGACY} ${RUNH2DIR}
rm ${RUNH2DIR}/${H2_ORIG}


# 3. Load a working backup into JVerein
# -------------------------------------

# Start JVEREIN and let the user import the "old" data base
echo 
echo '#########################################################################'
echo '### Please import backup manually using                               ###'
echo '###  "Datei|Backups verwalten|Anderes Backup wiederherstellen ..."    ###'
echo '### from within Jameica/JVerein!!!                                    ###'
echo '#########################################################################'
echo 
read -p 'Hit <RETURN> to continue ...' dummy
${RUNDIR}/jameica/jameica.sh &> /dev/null

# The backup will actually be imported when JVerein is started the next time
echo 
echo '#########################################################################'
echo '### Now, Jameica/JVerein will be restarted in order to import         ###'
echo '### the backup (using the legacy H2 driver)                           ###'
echo '### Please check if the import succeeded.                             ###'
echo '#########################################################################'
echo 
read -p 'Hit <RETURN> to continue ...' dummy
${RUNDIR}/jameica/jameica.sh &> /dev/null


# 4. Export the contents of the H2 database to an SQL file
# --------------------------------------------------------
# First save the database contents
mv ${TARGET_DATA} ${TARGET_DATA_BACKUP}

# Make sure that the user is informed about the backup
function message () {
echo '#########################################################################'
echo '### '
echo '### Done.'
echo '### '
echo '### An unchanged copy of the database has been located here:'
echo "###   ${TARGET_DATA_BACKUP}"
echo '### '
echo '### The directory and its contents can be removed if the conversion'
echo '### process succeeded.'
echo '### '
echo '### If something has gone wrong, the copy can be restored/copied'
echo '### manually to'
echo "###   ${TARGET_DATA}"
echo '### '
echo '#########################################################################'
echo 
}
trap "cleanup; message" EXIT

echo 
echo '#########################################################################'
echo '### Exporting the database (using the legacy H2 driver)               ###'
echo '#########################################################################'
# Export with legacy driver
java -cp ${LIBDIR}/${H2_LEGACY} org.h2.tools.Script -url jdbc:h2:${TARGET_DATA_BACKUP}/jverein \
  -user jverein -password jverein -script ${BACKUP}/export.sql


# 6. Import the contents from the SQL file into a new H2 database
# ---------------------------------------------------------------
echo 
echo '#########################################################################'
echo '### Importing the database (using the current H2 driver)              ###'
echo '#########################################################################'
mkdir ${TARGET_DATA}
# Import with original driver
java -cp ${LIBDIR}/${H2_ORIG} org.h2.tools.RunScript -url jdbc:h2:${TARGET_DATA}/jverein \
  -user jverein -password jverein -script ${BACKUP}/export.sql
  

# 7. Test with the actual Jameica installation
# --------------------------------------------
echo 
echo '#########################################################################'
echo '### Now the installed version of Jameica/JVerein will be started      ###'
echo '### (using the current H2 driver) ...                                 ###'
echo '### Please double-check that your data is complete and working!       ###'
echo '#########################################################################'
echo 
read -p 'Hit <RETURN> to continue ...' dummy
${TARGET_INSTALL}/jameica.sh &> /dev/null


exit 0
